!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     #########
SUBROUTINE VEGTYPE_GRID_TO_PATCH_GRID(KPATCH,PVEGTYPE_PATCH,PPATCH,PFIELDOUT,PW)
!        ################################################
!!
!!****  *VEGTYPE_GRID_TO_PATCH_GRID* averages fields from all (12) vegtypes 
!!                                   on only a few patches
!!    PURPOSE
!!    -------
!
!              
!!**  METHOD
!!    ------
!!
!!    REFERENCE
!!    ---------
!!
!!      
!!    AUTHOR
!!    ------
!!
!!      V. Masson          * Meteo-France *
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2004
!!
!-------------------------------------------------------------------------------

!
USE MODD_SURF_PAR,       ONLY : XUNDEF
USE MODD_DATA_COVER_PAR, ONLY : NVEGTYPE
!
USE MODI_VEGTYPE_TO_PATCH
!
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!
!*      0.1    declarations of arguments
!
INTEGER, INTENT(IN) :: KPATCH
REAL, DIMENSION(:,:,:), INTENT(IN)                          :: PVEGTYPE_PATCH
REAL, DIMENSION(:,:), INTENT(IN)                            :: PPATCH
REAL, DIMENSION(:,:,:), INTENT(IN)                          :: PFIELDOUT
REAL, DIMENSION(:,:,:), INTENT(OUT)                         :: PW
!
!
!*      0.2    declarations of local variables
!
REAL :: ZINT, ZINT2, ZINT3
REAL  :: ZPREC, ZPREC2
REAL, DIMENSION(SIZE(PW,1)) :: ZWI
REAL, DIMENSION(SIZE(PW,1),SIZE(PW,2),SIZE(PW,3)) :: ZW0
REAL, DIMENSION(SIZE(PW,1),SIZE(PW,3)) :: ZSUM
INTEGER                       :: JPATCH, JI, JP, JD   ! loop on patches
INTEGER                       :: JVEGTYPE  ! loop on vegtypes
INTEGER                       :: JLAYER    ! loop on layers
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
!---------------------------------------------------------------------------
!
!* averages from vegtypes to chosen number of patches
IF (LHOOK) CALL DR_HOOK('VEGTYPE_GRID_TO_PATCH_GRID',0,ZHOOK_HANDLE)
!
ZSUM(:,:) = 0.
DO JVEGTYPE=1,NVEGTYPE
  JPATCH = VEGTYPE_TO_PATCH(JVEGTYPE,KPATCH)
  DO JI = 1,SIZE(PW,1)
    ZSUM(JI,JPATCH) =ZSUM(JI,JPATCH) + PVEGTYPE_PATCH(JI,JVEGTYPE,JPATCH)
  ENDDO
ENDDO
!
PW(:,:,:) = 0.
ZW0(:,:,:) = 0.
DO JVEGTYPE=1,NVEGTYPE
  JPATCH = VEGTYPE_TO_PATCH(JVEGTYPE,KPATCH)
  DO JLAYER=1,SIZE(PW,2)
    ZWI(:) = PVEGTYPE_PATCH(:,JVEGTYPE,JPATCH) 
    ZWI(:) = ZWI(:) * PFIELDOUT(:,JLAYER,JVEGTYPE)
    ZW0(:,JLAYER,JPATCH) = ZW0(:,JLAYER,JPATCH) + ZWI(:)  
  END DO
END DO
!
PW(:,:,:) = ZW0(:,:,:)
!
DO JPATCH=1,SIZE(PW,3)
  DO JI = 1,SIZE(PW,1)
    IF (ZSUM(JI,JPATCH)/=0.) PW(JI,:,JPATCH) = PW(JI,:,JPATCH) / ZSUM(JI,JPATCH)
  ENDDO
ENDDO
!
ZPREC = 1.0E+16
!
DO JLAYER=1,SIZE(PW,2)
  DO JP = 1,SIZE(PW,3)
    DO JI = 1,SIZE(PW,1)
      IF (PW(JI,JLAYER,JP)/=XUNDEF) THEN
        ZPREC2 = 1.
        DO JD = 0,15
          IF (ABS(PW(JI,JLAYER,JP))<1./(10**(JD))) THEN
            ZPREC2 = ZPREC2*10.
          ELSE
            EXIT
          ENDIF
        ENDDO
        ZINT = AINT(PW(JI,JLAYER,JP))
        ZINT2 = AINT((PW(JI,JLAYER,JP)-ZINT)*ZPREC)
        ZINT3 = ANINT((PW(JI,JLAYER,JP)-ZINT-ZINT2/ZPREC)*ZPREC*ZPREC2)
        IF (PW(JI,JLAYER,JP)/=ZINT) THEN
          PW(JI,JLAYER,JP) = ZINT + ZINT2/ZPREC + ZINT3/ZPREC/ZPREC2
        ENDIF
      ENDIF
    ENDDO
  ENDDO
ENDDO
!
!* insures undefined value when patch is not present
DO JPATCH=1,KPATCH
  DO JLAYER=1,SIZE(PW,2)
    WHERE(PPATCH(:,JPATCH)==0.) PW(:,JLAYER,JPATCH) = XUNDEF
  END DO
END DO
WHERE( ABS(PW-XUNDEF)/XUNDEF < 1.E-6 ) PW = XUNDEF
IF (LHOOK) CALL DR_HOOK('VEGTYPE_GRID_TO_PATCH_GRID',1,ZHOOK_HANDLE)
!
!---------------------------------------------------------------------------
!
END SUBROUTINE VEGTYPE_GRID_TO_PATCH_GRID
